//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "MediaGroupViewController.h"
#import "ImageGridCell.h"
#import "PhotosVideosTabBarController.h"
#import "MusicTabBarController.h"
#import "PCFile+GridCell.h"
#import "PCResource+Info.h"
#import "PhotosVideosViewController.h"
#import "MusicTabBarController.h"
#import "TrackViewController.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+Dismiss.h"
#import "PCAccountInfo+additions.h"
#import "UIViewController+Spinner.h"
#import "ContactsDisplayViewController.h"
#import "OperationSpinner.h"
#import "OperationRefreshControl.h"
#import "ContactsTableViewController.h"
#import "PCContact+Additions.h"
#import "ImageDownloadView.h"

@interface ContactsDisplayViewController (Private)

@property (nonatomic, strong) NSArray *contacts;
@property (strong, nonatomic) NSIndexPath *selectedIndexPath;

@end

@interface MediaGroupViewController ()

@property (weak, nonatomic) IBOutlet UILabel *photosVideosEmptyTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *musicEmptyTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *contactEmptyTitleLabel;
@property (weak, nonatomic) IBOutlet UICollectionView *photosVideosGridView;
@property (weak, nonatomic) IBOutlet UICollectionView *musicGridView;
@property (weak, nonatomic) IBOutlet UICollectionView *contactsGridView;
@property (weak, nonatomic) IBOutlet OperationSpinner *photosVideosSpinner;
@property (weak, nonatomic) IBOutlet OperationSpinner *musicSpinner;
@property (weak, nonatomic) IBOutlet OperationSpinner *contactsSpinner;
@property (nonatomic, strong) NSArray *music;
@property (nonatomic, strong) NSArray *photos;
@property (nonatomic, strong) NSArray *videos;
@property (nonatomic, strong) NSArray *contacts;
@property (nonatomic, strong) OperationRefreshControl *operationalRefreshControl;

@end

@implementation MediaGroupViewController

- (void)viewDidLoad {
    [super viewDidLoad];
        
    self.navigationItem.title = NSLocalizedString(@"Home", nil);


    self.operationalRefreshControl = [[OperationRefreshControl alloc] init];
    [self.operationalRefreshControl addTarget:self action:@selector(reloadData) forControlEvents:UIControlEventValueChanged];
    [self.tableView addSubview:self.operationalRefreshControl];
    self.refreshControl = self.operationalRefreshControl;
    
    [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:searchRightBarAction], [NSNumber numberWithInt:shareRightBarAction], [NSNumber numberWithInt:refreshAccountBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];

    [self reloadData];
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {

    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case accountInfoRightBarAction: {
                    
                    PCAccountInfo *accountInfo = (PCAccountInfo *)result;
                    [self showOKAlertWithTitle:[self stringForRightBarAction:action]
                                    andMessage:[accountInfo humanReadableAccountInfo]
                              attributedString:nil
                                      okAction:nil];
                }
                    break;
                case refreshAccountBarAction: {
                    
                    [self showOKAlertWithTitle:NSLocalizedString(@"Success", nil) andMessage:NSLocalizedString(@"Successfully refreshed account. Please pull to refresh the screen", nil) attributedString:nil okAction:nil];
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }
}

#pragma mark -

- (void)fetchContacts {

    __weak MediaGroupViewController *blocksafeSelf = self;
    /*
     @Description :
     This API is used to fetch all contact details in an account. Once the fetch process is succeed/failed result will come with completion block.
     
     @Parameters: nil
      * For providing access token to SDK user have to logged In.
      * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.

     
     @CompletionHandler :
     This completion block gets called once contact fetch process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse :
     * Stores all the server response and error for this API.
     NSArray :
     * An array of PCContact object.
     
     
     @To fetch all contact details in an account, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
        #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCAPISDK object and call the fetchContactsWithCompletionHandler: API.
     * Perform the fetchContactsWithCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Sample code for fetchContactsWithCompletionHandler: API
    [[PCAPISDK sharedInstance] fetchContactsWithCompletionHandler:^(NSArray *contacts, PCAPIQueryResponse *response) {
        
        [blocksafeSelf.contactsSpinner removeOperation:@"contacts"];
        [blocksafeSelf.operationalRefreshControl removeOperation:@"contacts"];

        
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
            
                dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
                
                    NSSortDescriptor *contactSortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"firstName" ascending:YES selector:@selector(localizedCaseInsensitiveCompare:)];
                    blocksafeSelf.contacts = [contacts sortedArrayUsingDescriptors:[NSArray arrayWithObject:contactSortDescriptor]];
                    
                    dispatch_async(dispatch_get_main_queue(), ^(void) {
                    
                        if (blocksafeSelf.contacts.count) {
                            
                            blocksafeSelf.contactEmptyTitleLabel.hidden = YES;
                            [blocksafeSelf.contactsGridView reloadData];
                        } else {
                            
                            blocksafeSelf.contactEmptyTitleLabel.hidden = NO;
                        }
                        [blocksafeSelf.tableView reloadData];                    
                    });
                });
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [blocksafeSelf showUnAuthorizedAlert];
                [blocksafeSelf.tableView reloadData];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [blocksafeSelf showServerUnavailableAlert];
                [blocksafeSelf.tableView reloadData];
                break;
            default:
                [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Contacts", nil)];
                [blocksafeSelf.tableView reloadData];
                break;
        }
    }];
}

- (void)fetchAllFiles {
   
    __weak MediaGroupViewController *blocksafeSelf = self;
    
    [self.photosVideosSpinner addOperation:@"fullView"];
    [self.photosVideosSpinner addOperation:@"photoVideoSearch"];

    [self.musicSpinner addOperation:@"fullView"];
    [self.musicSpinner addOperation:@"musicSearch"];
    [self.contactsSpinner addOperation:@"contacts"];
    
    // Create the dispatch group
    dispatch_group_t fileGroup = dispatch_group_create();
    
    // Start the doFullSyncWithCompletionHandler: call.
    dispatch_group_enter(fileGroup);
    /*
     @Description :
     This API is used to refresh the local cache of files and folders. Based on X-Header-ETag value this API decides to do fullview or fullview with delta code internally. If Once the file cache is done/failed result will come with completion block.
     
     @Parameters : nil
     * accessToken are required parameter to call this API. For providing access token to SDK user have to logged In. If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     
     @CompletionHandler :
     This completion block gets called once the refresh of local cache is done/failed. completion block contains below parameter.
     PCAPIQueryResponse : 
     * Stores all the server response and error for this API.
     * For more information please check PCAPIQueryResponse.h class.
     
     @ To refresh the local cache of files and folders, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
        #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCAPISDK object and call the doFullSyncWithCompletionHandler: API.
     * Get the response(Success or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Sample code for doFullSyncWithCompletionHandler: API
    
    [[PCAPISDK sharedInstance] doFullSyncWithCompletionHandler:^(PCAPIQueryResponse *response) {
        
        [blocksafeSelf.photosVideosSpinner removeOperation:@"fullView"];
        [blocksafeSelf.musicSpinner removeOperation:@"fullView"];
        [blocksafeSelf.operationalRefreshControl removeOperation:@"fullView"];
        
        
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodePreconditionFailed:
            case PCAPIResponseCodeOK: {
                // Create the dispatch group
                dispatch_group_t fileGroup = dispatch_group_create();
                
                // Start the getPhoto call.
                dispatch_group_enter(fileGroup);
                
                /*
                 @Description :
                 This API is used to fetch all photos from the account. Once the fetch process is succeed/failed result will come with completion block.
                 
                 @Parameters: nil
                 * For providing access token to SDK user have to logged In.
                 * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                 
                 @CompletionHandler :
                 This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
                 PCAPIQueryResponse :
                 * Stores all the server response and error for this API.
                 NSArray :
                 * An array of PCPhoto object. PCPhoto class stores the metadata information of a photo files.
                 * Using PCPhoto object we can perform all FOPS operations Such as move,copy,rename and delete.
                 
                 @To fetch all photos from the account, you must take the following steps:
                 * Include VerizonCloudAPI.framework header file in your class.
                 #import <VerizonCloudAPI/VerizonCloudAPI.h>
                 * Initalize the PCAPISDK object and call the getPhotosWithCompletionHandler: API.
                 * Perform the getPhotosWithCompletionHandler: API call using the code sample below.
                 * Get the result (succeed or failed) in completion block.
                 * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                 */
                
                // Sample code for getPhotosWithCompletionHandler: API
          
                [[PCAPISDK sharedInstance] getPhotosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
                    dispatch_async(dispatch_get_main_queue(), ^(void) {
                        blocksafeSelf.photos = [NSArray arrayWithArray:itemList];
                        dispatch_group_leave(fileGroup);
                    });
                }];
                
                // Start the getVideo call.
                dispatch_group_enter(fileGroup);
            
                /*
                 @Description :
                 This API is used to fetch all videos from the account. Once the fetch process is succeed/failed result will come with completion block.
                 
                 @Parameters: nil
                 * For providing access token to SDK user have to logged In.
                 * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
                 
                 @CompletionHandler :
                 This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
                 PCAPIQueryResponse :
                 * Stores all the server response and error for this API.
                 NSArray :
                 * An array of PCVideo object. PCVideo class stores the metadata information of a video files.
                 * Using PCVideo object we can perform all FOPS operations Such as move,copy,rename and delete.
                 
                 @To fetch all videos from the account, you must take the following steps:
                 * Include VerizonCloudAPI.framework header file in your class.
                 #import <VerizonCloudAPI/VerizonCloudAPI.h>
                 * Initalize the PCAPISDK object and call the getVideosWithCompletionHandler: API.
                 * Perform the getVideosWithCompletionHandler: API call using the code sample below.
                 * Get the result (succeed or failed) in completion block.
                 * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                 */
                
                // Sample code for getVideosWithCompletionHandler: API

                [[PCAPISDK sharedInstance] getVideosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
                    dispatch_async(dispatch_get_main_queue(), ^(void) {
                        blocksafeSelf.videos = [NSArray arrayWithArray:itemList];
                        dispatch_group_leave(fileGroup);
                    });
                }];
                
                // Start the getAudio call.
                dispatch_group_enter(fileGroup);
                /*
                 @Description :
                 This API is used to fetch all audios from the account. Once the fetch process is succeed/failed result will come with completion block.
                 
                 @Parameters: nil
                 * For providing access token to SDK user have to logged In.
                 
                 @CompletionHandler :
                 This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
                 PCAPIQueryResponse :
                 * Stores all the server response and error for this API.
                 NSArray :
                 * An array of PCAudio object. PCAudio class stores the metadata information of a audio files.
                 * Using PCAudio object we can perform all FOPS operations Such as move,copy,rename and delete.
                 
                 @To fetch all audios from the account, you must take the following steps:
                 * Include VerizonCloudAPI.framework header file in your class.
                 #import <VerizonCloudAPI/VerizonCloudAPI.h>
                 * Initalize the PCAPISDK object and call the getAudiosWithCompletionHandler: API.
                 * Perform the getAudiosWithCompletionHandler: API call using the code sample below.
                 * Get the result (succeed or failed) in completion block.
                 * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
                 */
                
                // Sample code for getAudiosWithCompletionHandler: API

                [[PCAPISDK sharedInstance] getAudiosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
                    blocksafeSelf.music = [NSArray arrayWithArray:itemList];
                    dispatch_group_leave(fileGroup);

                }];
                
                // Notify group all calls completed.
                dispatch_group_notify(fileGroup,dispatch_get_main_queue(),^{
                    
                    [blocksafeSelf.photosVideosSpinner removeOperation:@"photoVideoSearch"];
                    [blocksafeSelf.operationalRefreshControl removeOperation:@"photoVideoSearch"];
                    
                    [blocksafeSelf.operationalRefreshControl removeOperation:@"musicSearch"];
                    [blocksafeSelf.musicSpinner removeOperation:@"musicSearch"];
                    
                    if (blocksafeSelf.photos.count + blocksafeSelf.videos.count) {
                        
                        blocksafeSelf.photosVideosEmptyTitleLabel.hidden = YES;
                        [blocksafeSelf.photosVideosGridView reloadData];
                        
                    } else {
                        blocksafeSelf.photosVideosEmptyTitleLabel.hidden = NO;
                    }
                    
                    if (blocksafeSelf.music.count) {
                        
                        blocksafeSelf.musicEmptyTitleLabel.hidden = YES;
                        [blocksafeSelf.musicGridView reloadData];
                        
                    } else {
                        blocksafeSelf.musicEmptyTitleLabel.hidden = NO;
                    }
                    
                    [blocksafeSelf.tableView reloadData];
                });
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [blocksafeSelf showUnAuthorizedAlert];
                [blocksafeSelf.tableView reloadData];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [blocksafeSelf showServerUnavailableAlert];
                [blocksafeSelf.tableView reloadData];
                break;
            default:
                [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Full view", nil)];
                [blocksafeSelf.tableView reloadData];
                break;
        }
        dispatch_group_leave(fileGroup);

    }];
    
    // Notify group all calls completed.
    dispatch_group_notify(fileGroup,dispatch_get_main_queue(),^{
        [self fetchContacts];
    });

}

- (void)reloadData {
    
    [self.operationalRefreshControl addOperation:@"contacts"];
    [self.operationalRefreshControl addOperation:@"fullView"];
    [self.operationalRefreshControl addOperation:@"photoVideoSearch"];
    [self.operationalRefreshControl addOperation:@"musicSearch"];

    [self fetchAllFiles];    
}

#pragma mark -
#pragma mark UICollectionViewDelegate

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    
    if (collectionView == self.photosVideosGridView) {
        return self.photos.count + self.videos.count;
    } else if (collectionView == self.musicGridView) {
        return self.music.count;
    } else if (collectionView == self.contactsGridView) {
        return self.contacts.count;
    }
    return 0;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    
    ImageGridCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"thumbnailCell" forIndexPath:indexPath];
    

    if (collectionView == self.photosVideosGridView || collectionView == self.musicGridView) {

        PCFile *resource;
        
        if (collectionView == self.photosVideosGridView) {
            
            if (indexPath.row < self.photos.count) {
                resource = [self.photos objectAtIndex:indexPath.row];
            } else {
                
                resource = [self.videos objectAtIndex:(indexPath.row - self.photos.count)];
            }
            
        } else if (collectionView == self.musicGridView) {
            
            resource = [self.music objectAtIndex:indexPath.row];
            
        }
        
        cell.titlelabel.text = resource.name;
        
        
        if ((resource.contentToken == cell.imageView.imageID) && cell.imageView.api) {
            // do nothing
        } else {
            
            [cell.imageView.api cancelAPI];
            
            [cell.spinner startAnimating];
            [resource setThumbnailForImageView:cell.imageView withCompletionHandler:^(BOOL didSetImage) {
                
                if (didSetImage) {
                    [cell.spinner stopAnimating];
                }
            }];
        }

    } else if (collectionView == self.contactsGridView) {
    
        PCContact *contactInfo = [self.contacts objectAtIndex:indexPath.row];
        cell.imageView.image = [UIImage imageNamed:@"Contact"];
        cell.titlelabel.text = [contactInfo getDisplaytext];
        
        if ((contactInfo.photoReference == cell.imageView.imageID) && cell.imageView.api) {
            // do nothing
        } else {
            
            [cell.imageView.api cancelAPI];
            [cell.spinner startAnimating];
            [contactInfo setAvatarForImageView:cell.imageView withCompletionHandler:^(BOOL didSetImage) {
                
                if (didSetImage) {
                    [cell.spinner stopAnimating];
                }
            }];
        }
    }
    
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    
    UIViewController *controller = nil;

    if (collectionView == self.photosVideosGridView || collectionView == self.musicGridView) {
        
        UITabBarController *tabBarController = nil;
        
        if (collectionView == self.photosVideosGridView) {
            tabBarController = (PhotosVideosTabBarController*)[storyBoard instantiateViewControllerWithIdentifier: @"PhotosVideosTabBarController"];
            UINavigationController *navigationController = [tabBarController.viewControllers objectAtIndex:0];
            PhotosVideosViewController *viewController = (PhotosVideosViewController *)[navigationController.viewControllers objectAtIndex:0];
            viewController.photosArray = self.photos;
            viewController.videosArray = self.videos;
            viewController.indexPath = indexPath;
        } else if (collectionView == self.musicGridView) {
            tabBarController = (MusicTabBarController*)[storyBoard instantiateViewControllerWithIdentifier: @"MusicTabBarController"];
            UINavigationController *navigationController = [tabBarController.viewControllers objectAtIndex:0];
            TrackViewController *viewController = (TrackViewController *)[navigationController.viewControllers objectAtIndex:0];
            viewController.dataSource = self.music;
        }
        
        controller = tabBarController;
        
    } else if (collectionView == self.contactsGridView) {
    
        UINavigationController *navigationController = nil;
        
        navigationController = [storyBoard instantiateViewControllerWithIdentifier:@"ContactsNavigationController"];
        ContactsDisplayViewController *contactsTableViewController = [navigationController.viewControllers objectAtIndex:0];
        contactsTableViewController.contacts = self.contacts;
        contactsTableViewController.selectedIndexPath = indexPath;
        
        controller = navigationController;
    }
    
    [self.navigationController presentViewController:controller animated:YES completion:nil];
}

#pragma mark -
#pragma UITableView methods

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger) section {

    NSString *header = nil;
    switch (section) {
        case 0: {
        
            if (self.photosVideosSpinner.numberOfOperations) {
                header = [NSString stringWithFormat:NSLocalizedString(@"Photos/Videos (Loading...)", nil)];
            } else {
               
                NSInteger totalPicsAndVideos = self.photos.count + self.videos.count;
                header = [NSString stringWithFormat:NSLocalizedString(@"Photos/Videos (%d)", nil), totalPicsAndVideos];
            }
        }
            break;
        case 1: {
        
            if (self.musicSpinner.numberOfOperations) {
                header = [NSString stringWithFormat:NSLocalizedString(@"Music (Loading...)", nil)];
            } else {
               
                NSInteger totalMusic = self.music.count;
                header = [NSString stringWithFormat:NSLocalizedString(@"Music (%d)", nil), totalMusic];
            }
        }
            break;
        case 2: {
            
            if (self.contactsSpinner.numberOfOperations) {
                header = [NSString stringWithFormat:NSLocalizedString(@"Contacts (Loading...)", nil)];
            } else {
                NSInteger totalContacts = self.contacts.count;
                header = [NSString stringWithFormat:NSLocalizedString(@"Contacts (%d)", nil), totalContacts];
            }
        }
            break;
    }
    return header;
}


- (void)dealloc {

    for (ImageGridCell *cell in self.photosVideosGridView.visibleCells) {
        
        [cell.imageView.api cancelAPI];
    }
    
    for (ImageGridCell *cell in self.musicGridView.visibleCells) {
        
        [cell.imageView.api cancelAPI];
    }
    
    for (ImageGridCell *cell in self.contactsGridView.visibleCells) {
        
        [cell.imageView.api cancelAPI];
    }
    
}


@end
