//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "MoveToPlaylistViewController.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+RightBarButtonAction.h"
#import "PCResource+Info.h"

@interface MoveToPlaylistViewController ()

@property (nonatomic, strong) NSArray *playlists;

@end

@implementation MoveToPlaylistViewController

@dynamic playlists;

- (void)viewDidLoad {
    [super viewDidLoad];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (playlistScreenType)playlistScreenType {

    if (self.selectedResource.fileType == imageFileType || self.selectedResource.fileType == videoFileType) {
        return playlistScreenTypeAlbum;
    } else if (self.selectedResource.fileType == musicFileType) {
        return playlistScreenTypeMusic;
    } else {
        return playlistScreenTypeUnknown;
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    PCPlaylist *playlist = [self.playlists objectAtIndex:indexPath.row];

    // Intialize the PCAddPlaylistItemsAPIParams object.
    PCAddPlaylistItemsAPIParams *params = [[PCAddPlaylistItemsAPIParams alloc] init];
    params.playlistUID = playlist.UID;
    params.paths = @[self.selectedResource.fullPath];
    
    __weak MoveToPlaylistViewController *blocksafeSelf = self;

    [self showSpinner];
    
    /*
     @Description :
     This API is used to add items in a given playlist. Once the adding process is succeed/failed result will come with completion block.
     
     @Parameters :
     PCAddPlaylistItemsAPIParams:
     * This class stores all the parameters required to add items in a given playlist. It contains attributes such as paths and playlistUID.
     * For providing access token to SDK user have to logged In.
     * paths and playlistUID are required parameter to call this API.
     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * More information about parameters you will find in PCAddPlaylistItemsAPIParams.h class.
     
     @CompletionHandler :
     This completion block gets called once the adding process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse : Stores all the server response and error for this API.
     
     @To add items in a given playlist, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCAddPlaylistItemsAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the  addPlaylistItems:withCompletionHandler: API.
     * Perform the  addPlaylistItems:withCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    // Initilize the PCAPISDK and call the API.
    
    
    [[PCAPISDK sharedInstance] addPlaylistItems:params withCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response) {
        // Get the result in this block.
        [blocksafeSelf hideSpinner];
        [blocksafeSelf dismissViewControllerAnimated:YES completion:^{
            blocksafeSelf.completionHandler(playlistItems, response);
        }];
    }];
}


@end
