//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCEditPlaylistAPITests : XCTestCase

@end

@implementation PCEditPlaylistAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (NSString*)valueBasedOnPlaylistType:(playlistType)playlistType {
    
    NSString *playlistTypeValue;
    
    switch (playlistType) {
        case playlistTypeMusic:
            playlistTypeValue = @"music";
            break;
        case playlistTypeImage:
            playlistTypeValue = @"image";
            break;
        case playlistTypeVideo:
            playlistTypeValue = @"video";
            break;
        case playlistTypeImageVideo:
            playlistTypeValue = @"image-video";
            break;
        case playlistTypeUnknown:
            break;
    }
    return playlistTypeValue;
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, addPlaylistItems api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCEditPlaylistAPIParams *params = [[PCEditPlaylistAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] editPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Edit playlist API with nil access token");
        
    }
    
}

/*!
 * Test Case:PlaylistName is required to edit playlist response,otherwise test will fail.
 */

- (void)testForRequiredPlaylistName {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
         XCTestExpectation *expectation = [self expectationWithDescription:@"Edit playlist API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                     XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        PCEditPlaylistAPIParams* params = [[PCEditPlaylistAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
                        
                        XCTAssertThrows([[PCAPISDK sharedInstance] editPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Edit playlist API without playlist name");
                    }
                    
                }
            break;
                    
                default:
                     XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {

            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

/*!
 * Test Case:PlaylistType is required to edit playlist response,otherwise test will fail.
 */

- (void)testForRequiredPlaylistType {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Edit playlist API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        PCEditPlaylistAPIParams* params = [[PCEditPlaylistAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        params.name = @"test rename Playlist";
                        
                        XCTAssertThrows([[PCAPISDK sharedInstance] editPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Edit playlist API without playlist type");
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

/*!
 * Test Case:PlaylistID is required to edit playlist response,otherwise test will fail.
 */

- (void)testForRequiredPlaylistID {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCEditPlaylistAPIParams* params = [[PCEditPlaylistAPIParams alloc] init];
        params.name = @"test rename Playlist";
        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] editPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {}],@"Edit playlist API without playlist ID");
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200, playlist object has to be of class PCPlaylist type. In all other cases (error), playlist has to be returned nil
 */

- (void)testToEditPlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Edit playlist API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        PCEditPlaylistAPIParams* params = [[PCEditPlaylistAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        params.name = @"test rename Playlist";
                        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
                        
                        [[PCAPISDK sharedInstance] editPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeOK:
                                    XCTAssertTrue([playlist isMemberOfClass:[PCPlaylist class]],@"Edit playlist API return PCPlaylist object");
                                    break;
                                default:
                                    XCTFail(@"Edit playlist API failed");
                                    break;
                            }
                            [expectation fulfill];
                        }];
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}


@end
