//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCThumbnailAPITests : XCTestCase

@end

@implementation PCThumbnailAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCThumbnailAPIParams *params = [[PCThumbnailAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] downloadThumbnail:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with nil access token");
        
    }
}

/*!
 * Test Case: query is a required parameter. SDK should throw an exception in case query is nil.
 */
- (void)testNilContentToken {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCThumbnailAPIParams *params = [[PCThumbnailAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] downloadThumbnail:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with nil content token");
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: Either 'size' or 'th' and 'tw' are required. In all other cases SDK should throw an exception.
 */
- (void)testSizeThTwCombination {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCThumbnailAPIParams *params1 = [[PCThumbnailAPIParams alloc] init];
        params1.contentToken = @"testContentToken";
        
        XCTAssertThrows([[PCAPISDK sharedInstance] downloadThumbnail:params1 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with Size , Th and Tw absent");
        
        PCThumbnailAPIParams *params2 = [[PCThumbnailAPIParams alloc] init];
        params2.contentToken = @"testContentToken";
        params2.size = @"m";
        
        XCTAssertNoThrow([[PCAPISDK sharedInstance] downloadThumbnail:params2 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with Size present");
        
        PCThumbnailAPIParams *params3 = [[PCThumbnailAPIParams alloc] init];
        params3.contentToken = @"testContentToken";
        params3.th = 128.0;
        
        XCTAssertThrows([[PCAPISDK sharedInstance] downloadThumbnail:params3 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with th present but no tw");
        
        PCThumbnailAPIParams *params4 = [[PCThumbnailAPIParams alloc] init];
        params4.contentToken = @"testContentToken";
        params4.tw = 128.0;
        
        XCTAssertThrows([[PCAPISDK sharedInstance] downloadThumbnail:params4 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with tw present but no th");
        
        PCThumbnailAPIParams *params5 = [[PCThumbnailAPIParams alloc] init];
        params5.contentToken = @"testContentToken";
        params5.th = 128.0;
        params5.tw = 128.0;
        
        XCTAssertNoThrow([[PCAPISDK sharedInstance] downloadThumbnail:params5 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with tw and th present");
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}


/*!
 * Test Case: 'xs' 's' and 'm' are valid values for size. Assigning any other value should throw an exception.
 */
- (void)testValidValuesOfSize {

    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCThumbnailAPIParams *params1 = [[PCThumbnailAPIParams alloc] init];
        params1.contentToken = @"testContentToken";
        params1.size = @"a";
        
        XCTAssertThrows([[PCAPISDK sharedInstance] downloadThumbnail:params1 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with Size Th and Tw absent");
        
        PCThumbnailAPIParams *params2 = [[PCThumbnailAPIParams alloc] init];
        params2.contentToken = @"testContentToken";
        params2.size = @"s";
        
        XCTAssertNoThrow([[PCAPISDK sharedInstance] downloadThumbnail:params2 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with Size Th and Tw absent");
        
        PCThumbnailAPIParams *params3 = [[PCThumbnailAPIParams alloc] init];
        params3.contentToken = @"testContentToken";
        params3.size = @"xs";
        
        XCTAssertNoThrow([[PCAPISDK sharedInstance] downloadThumbnail:params3 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with Size Th and Tw absent");
        
        PCThumbnailAPIParams *params4 = [[PCThumbnailAPIParams alloc] init];
        params4.contentToken = @"testContentToken";
        params4.size = @"m";
        
        XCTAssertNoThrow([[PCAPISDK sharedInstance] downloadThumbnail:params4 withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Thumbnail API with Size Th and Tw absent");
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: image has to be nil, if in any case download fails. Corrupted image should not be returned.
 * Using search pick a file out of DB.
 * Download thumbnail.
 * If response status code is not 200 (error case), image returned has to be nil.
 */
- (void)testThumbnailAPIResponse {

    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Download thumbnail API Result"];
        
        PCSearchAPIParams *photoSearchParams = [[PCSearchAPIParams alloc] init];
        photoSearchParams.query = [NSString setValue:@"image*" forKey:@"contentType"];
        [photoSearchParams setSortKey:@"name" isAscending:YES];
        
        [[PCAPISDK sharedInstance] search:photoSearchParams withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {
            if (results.count) {
                
                    NSManagedObjectID *objID = [results objectAtIndex:0];
                    PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                    
                    PCThumbnailAPIParams *params = [[PCThumbnailAPIParams alloc] init];
                    params.contentToken = resource.contentToken;
                    params.size = @"m";
                    
                    [[PCAPISDK sharedInstance] downloadThumbnail:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                        
                        NSData *fileData    = [NSData dataWithContentsOfURL:response.downloadedURL];
                        UIImage *image      = [UIImage imageWithData:fileData];
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeOK: {
                                XCTAssertNotNil(image, @"Thumbnail API response image not nil");
                            }
                                break;
                            default: {
                                
                                XCTAssertNil(image, @"Thumbnail API response image nil");
                            }
                                break;
                        }
                        [expectation fulfill];
                    }];
                
            } else {
                 XCTFail(@"Test could not be completed");
            }
            [expectation fulfill];
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

@end
