//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "ImageViewerViewController.h"
#import "UIViewController+Alert.h"
#import "UIViewController+Spinner.h"
#import "PCResource+Info.h"
#import "ZoomableImageGridCell.h"
#import "PCResource+GridCell.h"
#import "UIViewController+ResourceActions.h"
#import "UIViewController+RightBarButtonAction.h"
#import "ImageDownloadView.h"
#ifdef KEPLER_FILE_BROWSER
#import "MetaDataViewController.h"
#endif
@interface ImageViewerViewController () <ImageGridCellDelegate>

@property (nonatomic, strong) PCAPI *filesAPI;
@property (weak, nonatomic) IBOutlet UICollectionView *galleryView;

@end

@implementation ImageViewerViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    
    [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:signOutRightBarAction], nil];
    
    self.galleryView.allowsSelection = YES;
    
    [self.galleryView layoutIfNeeded];
    [self.galleryView scrollToItemAtIndexPath:[NSIndexPath indexPathForRow:self.selectedIndex inSection:0] atScrollPosition:UICollectionViewScrollPositionCenteredHorizontally animated:NO];
}

- (void)didFinishDoingAction:(ResourceAction)action onResource:(id)resource result:(id)result response:(id)response {
    switch (action) {
        case addToGalleryResourceAction: {

            self.navigationItem.rightBarButtonItem.enabled = YES;

            NSError *error = (NSError *)response;
            if (error) {
                [self showOKAlertWithTitle:@"Failed" andMessage:NSLocalizedString(@"Failed to save image to gallery", nil) attributedString:nil okAction:nil];
            } else {
                [self showOKAlertWithTitle:@"Success" andMessage:NSLocalizedString(@"image saved to gallery", nil) attributedString:nil okAction:nil];
            }

        }
            break;
            
        default:
            break;
    }
}

- (id)dataSourceForResourceAction:(ResourceAction)action {

    switch (action) {
        case addToGalleryResourceAction: {
        
            NSArray *visibleCells = [self.galleryView visibleCells];
            if (visibleCells.count) {
                ImageGridCell *currentCell = [visibleCells objectAtIndex:0];
                return currentCell.imageView.image;
            }
            return nil;
        }
            break;
            
        default:
            return nil;
            break;
    }
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.dataSource.count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    
    ZoomableImageGridCell *cell = (ZoomableImageGridCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"ZoomableImageGridCell" forIndexPath:indexPath];
    
    PCResource *resource = [self.dataSource objectAtIndex:indexPath.row];
    self.title = resource.name;
    
    cell.gridCellDelegate = self;
    
    if ((resource.contentToken == cell.imageView.imageID) && cell.imageView.api) {
        // do nothing
    } else {
    
        [cell.imageView.api cancelAPI];
        
        [cell.spinner startAnimating];
        cell.progressView.hidden = NO;
        cell.progressBar.progress = 0;
        
        [resource setFullResolutionImageView:cell.imageView withProgressHandler:^(float progress) {
            
            cell.progressBar.progress = progress;

        } andCompletionHandler:^(BOOL didSetImage) {
            
            if (didSetImage) {
                [cell.spinner stopAnimating];
                cell.progressView.hidden = YES;
            }
        }];
    }
   
    return cell;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    
    CGSize size = self.galleryView.bounds.size;
    ZoomableImageGridCell *cell = (ZoomableImageGridCell *)[collectionView cellForItemAtIndexPath:indexPath];
    cell.scrollView.contentSize = size;
    
    return size;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    
    PCResource *resource = [self.dataSource objectAtIndex:indexPath.row];
    
    if (![[resource file] boolValue]) {
        
#ifdef KEPLER_FILE_BROWSER
        UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
        
        MetaDataViewController *controller = (MetaDataViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"MetaDataViewController"];        
        controller.rootResource = resource;
        [self.navigationController pushViewController:controller animated:YES];
#endif

    }
}



#pragma mark -

- (void)gridCellDidLongPressed:(ImageGridCell *)cell {

    
    NSIndexPath *selectedIndexPath = [self.galleryView indexPathForCell:cell];
    
    PCResource *resource = [self.dataSource objectAtIndex:selectedIndexPath.row];
    if (resource.fileType == imageFileType) {
        
        if (cell.imageView.image) {
            
            // passing resource is not needed. image is retrived through datasource method
            [self resourceSelected:nil showActions:[NSNumber numberWithInt:addToGalleryResourceAction], nil];
        } else {
            [self showOKAlertWithTitle:NSLocalizedString(@"Please wait", nil) andMessage:NSLocalizedString(@"Please wait while image is getting downloaded", nil) attributedString:nil okAction:nil];
        }
    }
}

- (void)gridCellDidSelected:(ImageGridCell *)cell {

    NSIndexPath *selectedIndexPath = [self.galleryView indexPathForCell:cell];
    [self collectionView:self.galleryView didSelectItemAtIndexPath:selectedIndexPath];
}

#pragma mark -

- (void)dealloc {
    
    for (ImageGridCell *cell in self.galleryView.visibleCells) {
        
        [cell.imageView.api cancelAPI];
    }
}


@end
