//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "MedialistViewController.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+ResourceActions.h"
#import "UIViewController+Spinner.h"
#import "NSArray+Addition.h"
#import "UploadManager.h"
#import "ProgressView.h"

@interface FileGridViewController (Protected)

@property (nonatomic, strong) NSString *emptyViewMessage;
@property (nonatomic, retain) UIRefreshControl *refreshControl;

- (void)refreshView;
- (void)alertController:(UIAlertController *)alertController
     addActionWithTitle:(NSString *)title
                  style:(UIAlertActionStyle)style
              andAction:(void (^)(UIAlertAction * _Nonnull action))handler;

@end

@interface MedialistViewController ()

@property (weak, nonatomic) IBOutlet ProgressView *progressView;
@property (nonatomic ,assign) rightBarAction rightBarAction;

@end

@implementation MedialistViewController

- (void)reloadData {
    
    switch (self.rightBarAction) {
            
        case photoMediaRightBarAction:
            self.title = NSLocalizedString(@"Photo", nil);

            [self callGetPhotoAPI];
            break;
            
        case audioMediaRightBarAction:
            self.title = NSLocalizedString(@"Audio", nil);

            [self callGetAudioAPI];
            break;
            
        case videoMediaRightBarAction:
            self.title = NSLocalizedString(@"Video", nil);

            [self callGetVideoAPI];
            break;
            
        case docMediaRightBarAction:
            self.title = NSLocalizedString(@"Document", nil);
            
            [self callGetDocAPI];
            break;
            
        default:
            break;
    }

}

#pragma mark
#pragma Call data categories methods.

- (void) callGetPhotoAPI
{
    __weak MedialistViewController *blocksafeSelf = self;
    
    // Sample code for getPhotosWithCompletionHandler: API
    [[PCAPISDK sharedInstance] getPhotosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];
        blocksafeSelf.dataSource = itemList;
        [blocksafeSelf refreshView];
    }];
}

- (void) callGetAudioAPI
{
    __weak MedialistViewController *blocksafeSelf = self;
    
    // Sample code for getAudiosWithCompletionHandler: API
    
    [[PCAPISDK sharedInstance] getAudiosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];
        blocksafeSelf.dataSource = itemList;
        [blocksafeSelf refreshView];
    }];
}

- (void) callGetVideoAPI
{
    __weak MedialistViewController *blocksafeSelf = self;
    
    // Sample code for getVideosWithCompletionHandler: API
    
    [[PCAPISDK sharedInstance] getVideosWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];
         blocksafeSelf.dataSource = itemList;
        [blocksafeSelf refreshView];
    }];
}

- (void) callGetDocAPI
{
    __weak MedialistViewController *blocksafeSelf = self;
    
    // Sample code for getDocumentsWithCompletionHandler: API

    [[PCAPISDK sharedInstance] getDocumentsWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];
        blocksafeSelf.dataSource = itemList;
        [blocksafeSelf refreshView];
    }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = NSLocalizedString(@"Media Type", nil);
    self.rightBarAction = photoMediaRightBarAction;
    self.emptyViewMessage = NSLocalizedString(@"Folder is empty", nil);
    
    [self showSpinner];
    [self.refreshControl beginRefreshing];

    [self reloadData];
    

}

- (void)viewWillAppear:(BOOL)animated {

    [super viewWillAppear:animated];
    
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    [self showSpinner];
    self.dataSource = nil;
    self.rightBarAction = action;
    
    switch (self.rightBarAction) {
        case photoMediaRightBarAction: {
            self.title = NSLocalizedString(@"Photo", nil);
        }
            break;
        case videoMediaRightBarAction: {
            self.title = NSLocalizedString(@"Video", nil);
        }
            break;
        case audioMediaRightBarAction: {
            self.title = NSLocalizedString(@"Audio", nil);
        }
            break;
        case docMediaRightBarAction: {
            self.title = NSLocalizedString(@"Document", nil);
        }
            break;
        case accountInfoRightBarAction:
        case searchRightBarAction:
        case createFolderRightBarAction:
        case signOutRightBarAction:
        case uploadFileRightBarAction:
        case emptyTrashRightBarAction:
        case refreshAccountBarAction:
            break;
    }
}


- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    self.dataSource = (NSArray *)result;
    [self refreshView];
 
}

- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    return nil;
}


#pragma mark -


- (void)didBeginDoingResourceAction:(ResourceAction)action {
    [self showSpinner];
}

- (void)didFinishDoingAction:(ResourceAction)action onResource:(id)resource result:(id)result response:(id)response {
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    [self hideSpinner];
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeNoContent:
        case PCAPIResponseCodeOK: {
            
            switch (action) {
#ifdef KEPLER_FILE_BROWSER
                    
                case moveResourceAction:
                case copyResourceAction:
                    [self showSpinner];
                    [self reloadData];
                    break;
                case restoreResourceAction:
                case deleteResourceAction: {
                    
                    NSMutableArray *mutableDataSource = [NSMutableArray arrayWithArray:self.dataSource];
                    [mutableDataSource removeObject:resource];
                    self.dataSource = [NSArray arrayWithArray:mutableDataSource];
                    [self refreshView];
                }
                    break;
                case renameResourceAction: {
                    
                    if ([self isKindOfClass:[MedialistViewController class]]) {
                        PCResourceItem *resourcItem = (PCResourceItem *) result;
                        self.dataSource = [self.dataSource arrayByReplacingObject:resource withObject:resourcItem andSortByKeyPath:nil ascending:NO];
                    }
                    
                    [self refreshView];
                }
                    break;
#endif
#if HUBBLE_MEDIA_BROWSER
                case addToAlbumResourceAction:
                case fileInfoResourceAction:
                case addToPlaylistResourceAction:
                    break;
                case removeFromAlbumResourceAction:
                case removeFromPlaylistResourceAction: {
                    
                    NSMutableArray *mutableDataSource = [NSMutableArray arrayWithArray:self.dataSource];
                    [mutableDataSource removeObject:resource];
                    self.dataSource = [NSArray arrayWithArray:mutableDataSource];
                    [self refreshView];
                    
                }
                default:
                    break;
#endif
            }
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForResourceAction:action]];
            break;
    }
}


@end
