//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCDeletePathAPITests : XCTestCase

@end

@implementation PCDeletePathAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, copy should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCDeleteParams *params = [[PCDeleteParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] deletePath:params withCompletionHandler:^(PCAPIQueryResponse *response){}] ,@"Delete with nil access token");
    }
}

/*!
 * Test Case: Whenever PCDeleteParams is allocated default value of purge needs to be false. Test fails otherwise.
 */
- (void)testDefaultParameterValue {
    
    PCDeleteParams *params = [[PCDeleteParams alloc] init];
    XCTAssertEqual(params.purge, false);
}

/*!
 * Test Case: path is a required parameter. SDK should throw an exception in case path is nil.
 */
- (void)testValidPathForDeleteAPI {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCDeleteParams *params = [[PCDeleteParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] deletePath:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Delete without valid path");
         
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200,  delete API response contains the file which copied.
 */
- (void)testToDeleteFileToGivenPath {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test to delete file using delete path API"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCDeleteParams *params = [[PCDeleteParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.path = resource.fullPath;
                    
                    [[PCAPISDK sharedInstance] deletePath:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeOK:
                                break;
                            default:
                                 XCTFail(@"Test Failed file not deleted");
                                break;
                        }
                        [expectation fulfill];
                    }];
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }

    
}

@end
