//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"


#define MAX_FILE_SIZE_FOR_NON_CHUNK_UPLOAD  10 * 1024 * 1024

@interface PCUploadAPITests : XCTestCase

@end

@implementation PCUploadAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, copy should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCUploadAPIParams *params = [[PCUploadAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] uploadFile:params readFromPath:nil progress:^(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded) {} withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}],@"Upload with nil access token");
        
    }
}

- (NSString*) returnLocalPathForWithoutChunkUpload {
    NSError *error;
    NSString *stringToWrite = @"1\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n15";
    
    NSString *filePath = [[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) firstObject] stringByAppendingPathComponent:@"fileUpload.txt"];
    
    [stringToWrite writeToFile:filePath atomically:YES encoding:NSUTF8StringEncoding error:&error];
    
    return filePath;
}

- (NSString*) returnLocalPathForWithChunkUpload {
    NSError *error;
    
    NSMutableString *bigString = [NSMutableString string];
    
     NSString *stringToWrite = @"1\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n151\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n15";
    
    for (int i=0; i<20000; i++) {
        [bigString appendString:stringToWrite];
    }
   
    NSString *filePath = [[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) firstObject] stringByAppendingPathComponent:@"fileUpload.txt"];
    
    [bigString writeToFile:filePath atomically:YES encoding:NSUTF8StringEncoding error:&error];
    
    return filePath;
}

/*!
 * Test Case:Local path of the file is required parameter. SDK should throw an exception in case local path of the file is nil.
 */

- (void)testValidLocalPath {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCUploadAPIParams *params = [[PCUploadAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] uploadFile:params readFromPath:nil progress:^(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded) {} withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}],@"Upload without valid local path");
    } else {
         XCTFail(@"Test Failed due to nil access token");
    }
    
}

/*!
 * Test Case: The path to the folder where file has to be uploaded is required parameter. SDK should throw an exception in case path to the folder is nil.
 */

- (void)testValidTargetPathToUpload {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCUploadAPIParams *params = [[PCUploadAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] uploadFile:params readFromPath:[self returnLocalPathForWithoutChunkUpload] progress:^(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded) {} withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}],@"Upload without valid target path");
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: The name of file has to be uploaded is required parameter. SDK should throw an exception in case file name is nil.
 */

- (void)testValidfileNameToUpload {
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCUploadAPIParams *params = [[PCUploadAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        XCTAssertThrows([[PCAPISDK sharedInstance] uploadFile:params readFromPath:[self returnLocalPathForWithoutChunkUpload] progress:^(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded) {} withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}],@"Upload without valid file name");
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: Check file with less than MAX_FILE_SIZE_FOR_NON_CHUNK_UPLOAD , will upload the file without chunking.
 */

-(void) testFileToUploadWithoutChunk {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
         XCTestExpectation *expectation = [self expectationWithDescription:@"Test file upload without chunk"];
        
        PCUploadAPIParams *params = [[PCUploadAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        params.name = @"upload_without_chunk";
        
        NSString *filePath = [self returnLocalPathForWithoutChunkUpload];
        
        [[PCAPISDK sharedInstance] uploadFile:params readFromPath:filePath progress:^(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded) {
            
        } withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
            
            if (response.urlResponse.statusCode == PCAPIResponseCodeResourceCreated || response.urlResponse.statusCode == PCAPIResponseCodeOK) {
                XCTAssertTrue([metadata isMemberOfClass:[PCMetadata class]], @"File uploaded successfully without chunk");
            }else {
               XCTFail(@"Failed to upload file without chunk");
            }
            [[NSFileManager defaultManager] removeItemAtPath:filePath error:nil];
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
        
    }else {
       XCTFail(@"Test Failed due to nil access token");
    }
    
}


/*!
 * Test Case: Check file with more than MAX_FILE_SIZE_FOR_NON_CHUNK_UPLOAD , will upload the file with chunking.
 */

-(void) testFileToUploadWithChunk {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test file upload with chunk"];
        
        PCUploadAPIParams *params = [[PCUploadAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        params.name = @"upload_with_chunk";
        
        NSString *filePath = [self returnLocalPathForWithChunkUpload];
        
        [[PCAPISDK sharedInstance] uploadFile:params readFromPath:filePath progress:^(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded) {
            
        } withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
            
            if (response.urlResponse.statusCode == PCAPIResponseCodeResourceCreated || response.urlResponse.statusCode == PCAPIResponseCodeOK) {
                XCTAssertTrue([metadata isMemberOfClass:[PCMetadata class]], @"File uploaded successfully with chunk");
            }else {
                XCTFail(@"Failed to upload file without chunk");
            }
            [[NSFileManager defaultManager] removeItemAtPath:filePath error:nil];
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*7 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
        
    }else {
        XCTFail(@"Test Failed due to nil access token");
    }
    
}



@end
