//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>
#import "PCResourceItem.h"


/*!
 * This class stores the metadata information of a file.
 */

@interface PCFile : PCResourceItem

/*!
 * Specifies the content type of the file.
 */
@property (nonatomic, readonly, copy) NSString *mimeType;

/*!
 * The date, in UTC format, that the file was created on the client system. For example, where a desktop client uploads a photo originally taken on a camera, Cre- ation-Date will differ from Capture-Date, while on a camera phone the two attributes are likely to match.
 */
@property (nonatomic, readonly, strong) NSDate *creationDate;


/*!
 * The city, town, village or general locality.
 */
@property (nonatomic, readonly, copy) NSString *locationLocality;

/*!
 * The county.
 */
@property (nonatomic, readonly, copy) NSString *locationCountry;

/*!
 * Dictates where a media file should appear in a time- line view on a client application.
 */
@property (nonatomic, readonly, strong) NSDate *timelineDate;

/*!
 * If true, the resource is a file.
 */
@property (nonatomic, readonly, strong) NSNumber *file;

/*!
 * Call this methods to create a PCFile using a managedObjectID. managedObjectID is mapped with coredata object and a PCResource is created out of the values copied from the coredata object.
 */
- (id)initWithDBResourceID:(NSManagedObjectID *)managedObjectID;

- (id)initWithDict:(NSDictionary *)dict;


@end
